/* -*- Mode: Java; tab-width: 2; c-basic-offset: 2; -*-
 * 
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation.  All Rights
 * Reserved.
 *
 * Contributor(s):
 *  Doug Daniels <rainking@rice.edu> -- morphed code for Helper Applications pref window
 *                                      into multiple servers pref window for Annozilla
 */

const SERVERS_ROOT_URN = "urn:annotations:servers";
const SERVERS_FILENAME = "annozillaServers.rdf";

var gRDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
var gDS = null;

var gServerURLstr = null;
var gUser= null;
var gPassword = null;


/**
 * Initializes the window, loading
 * information about the server from
 * the datasource. Takes the server URL String
 * as argument[0] to the window.
 */
function Startup()
{
	// set up window buttons

  doSetOKCancel(onOK);

	// get the server URL
  if (window.arguments) {      
      gServerURLstr = window.arguments[0];
  }
	else {
		throw new Error("Edit server window called without proper arguments -- no serverURL");
	}

  gUser  = document.getElementById("user");
  gPassword  = document.getElementById("password");
    
  gUser.focus();
  
  sizeToContent();
  moveToAlertPosition();

  // load the data source and set the values
  var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);  
  var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
  file.append(SERVERS_FILENAME);  
	var urlSpec = getURLSpecFromFile(file);
  gDS = gRDF.GetDataSourceBlocking(urlSpec);    

  // get list element for this server
  var element = gRDF.GetResource(gServerURLstr);

  gUser.value = getLiteralFromResource(element, "user");
  gPassword.value = getLiteralFromResource(element, "password");  
  document.getElementById ("confirmation").value = getLiteralFromResource(element, "password");  
  
  // set server name label
  document.getElementById("serverName").value = gServerURLstr;
}


/**
 * Called upon pressing OK.  Updates the datasource
 * with the user's input.
 */
function onOK()
{

    if ( !passwordsMatch() ) {
        window.alert("Password confirmation does not match password!"); // TODO i18n
        return false;
    }

	// get the datasource
	var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);  
    var file = fileLocator.get("ProfD", Components.interfaces.nsIFile);
    file.append(SERVERS_FILENAME);
		var urlSpec = getURLSpecFromFile(file);
    gDS = gRDF.GetDataSourceBlocking(urlSpec);    

    // get the server and change the user and password
    var element = gRDF.GetResource(gServerURLstr);
    changeLiteralForResource(element, "user", gUser.value);
    changeLiteralForResource(element, "password", gPassword.value);
		
    // flush the ds to disk.   
    var remoteDS = gDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteDS.Flush();

    window.close();  
}

/**
 * Prepend the server namespace string to a property.
 *
 * @param aProperty String property
 * @return concatenated String
 */
function SERVER_RDF(aProperty)
{
    return "urn:annotations:serverNS#" + aProperty;
}

/**
 * Change a literal for a resource in the global
 * datasource.
 *
 * @param aResource Resource to change the literal for
 * @param aPropertyString literal to change
 * @param aValueString value for that literal
 */
function changeLiteralForResource(aResource, aPropertyString, aValueString)
{
    var valueProperty = gRDF.GetResource(SERVER_RDF(aPropertyString));
    var newLiteral = gRDF.GetLiteral(aValueString);
    var currentValue = gDS.GetTarget(aResource, valueProperty, true);
    gDS.Change(aResource, valueProperty, currentValue, newLiteral);
}

/**
 * Get a literal's String value from a resource in the global
 * datasource.
 *
 * @param aResource Resource to change the literal for
 * @param aPropertyString literal to get
 * @return the String value of that literal
 */
function getLiteralFromResource(aResource, aPropertyString) {
    var valueProperty = gRDF.GetResource(SERVER_RDF(aPropertyString));
    var node = gDS.GetTarget(aResource, valueProperty, true);
    node = node.QueryInterface(Components.interfaces.nsIRDFLiteral);
    return node.Value;
}

/**
 * Check that the password and confirmation match.
 */
function passwordsMatch () {
    return (document.getElementById ("password").value ==
            document.getElementById ("confirmation").value);
}

function getURLSpecFromFile(file) {
    
    if (Components.classes["@mozilla.org/network/io-service;1"] && Components.interfaces.nsIIOService) {
	
	var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);

	// new version
	if (fileHandler.getURLSpecFromFile) {
	    return fileHandler.getURLSpecFromFile(file);
	}
	// old version in ioService
	else {
	    return ioService.getURLSpecFromFile(file);
	}
    }
    // really old version
    else {
	var file_url = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIFileURL);
	file_url.file = file;
	return file_url.spec;
    }
}
